# Generated by roxygen2: do not edit by hand

export(fbam)
export(get_est)
export(get_plot_data)
export(hbam)
export(hbam_cv)
export(plot_by_group)
export(plot_over_self)
export(plot_respondents)
export(plot_stimuli)
export(prep_data)
export(prep_data_cv)
import(RColorBrewer)
import(Rcpp)
import(ggplot2)
import(loo)
import(methods)
import(parallel)
import(stats)
import(tidyr)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(colorspace,darken)
importFrom(dplyr,bind_cols)
importFrom(future,multisession)
importFrom(future,plan)
importFrom(future,sequential)
importFrom(future.apply,future_lapply)
importFrom(matrixStats,colLogSumExps)
importFrom(plyr,llply)
importFrom(rlang,.data)
importFrom(rstan,sampling)
importFrom(rstan,sflist2stanfit)
importFrom(rstantools,rstan_config)
useDynLib(hbamr, .registration = TRUE)
