#' 2012 Liberal-Conservative Scales
#'
#' Liberal-Conservative 7-point scales from the 2012 National Election Study. Includes
#' (in order) original case id, self-placement, and rankings of Obama, Romney, Democratic Party,
#' Republican party. Missing values are coded as NA.
#'
#' @docType data
#'
#' @usage data(LC2012)
#'
#' @keywords datasets
#'
#' @source American National Election Studies.
#'
#' @examples
#' data(LC2012)
#' head(LC2012)
"LC2012"
